/* Sample code RectFlurry by Rudy Rucker, CS Dept, SJSU, August 28, 2003.
Expanded from F.S.Hill's COMPUTER GRAPHICS book sample code for Chapter 2,
Figure 2_31. This code is public domain and can be freely reused. */

#include <windows.h>   // use as needed for your system
#include <time.h> //For the clock() function used by sleep() for animation.
#include <gl\gl.h> //Note that backslashes have to be like \, not / for Windows.
#include <gl\glu.h>
#include <gl\glut.h> //You may need to download this from www.opengl.org

/* Note that you may need to add glu32.lib glut32.lib opengl32.lib to your
link list in Project Settings | Link | Object/Library Modules. 
Also note that you may need to add glut32.dll to your Windows SYSTEM32 directory. */


GLdouble MAXX = 640.0;
GLdouble MAXY = 480.0;
GLdouble TIMEPERFRAME = 0.5; //Time in seconds per frame
//#define AUTOANIMATE
/* If you comment in AUTOANIMATE and build the program, then it randomizes
the image automatically every TIMEPERFRAME seconds. */

//<<<<<<<<<<<<<<<<<<<<<<< myInit >>>>>>>>>>>>>>>>>>>>
void myInit(void)
{
/* We call myInit once at startup. */
    glClearColor(1.0,1.0,1.0,0.0);       // set white background color
    glColor3f(0.0f, 0.0f, 0.0f);          // set the drawing color 
 	glPointSize(4.0);		       // a dot is 4 by 4 pixels
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	gluOrtho2D(0.0, MAXX, 0.0, MAXY);
}
//<<<<<<<<<<<<<<<<<<<<<<<< utilities >>>>>>>>>>>>>>>>>
int random(int m)
{
	return rand()%m; /* The C rand function returns a random
		int between 0 and BIGNUM (about 2 billion when using 32 bit int).
		We use the % modulo method to get a number less than m so
		that our random(m) returns a number from 0 through m-1. */
}

GLdouble randomdouble()
{
/* returns a number between 0.0 and 1.0 */
	return GLdouble(random(1000))/1000.0;
}

GLdouble random(GLdouble f)
{
/* returns a number between 0.0 and f */
	return randomdouble()*f;
}

void sleep(double secondstowait)
{
/* This makes the program spin in a loop for an interval of secondstowait.
It can make functionality a bit awkward for long secondstowait, as you aren't
polling the keyboard and mouse while sleep spins, which can make the
program feel dead.*/
	long waitticks = (long)(secondstowait * CLOCKS_PER_SEC);
		//CLOCKS_PER_SEC is typically 30, it's a system thing.
	long goal = waitticks + clock();
	while( goal > clock() )
      ; //Spin a few thousand times.
}

//<<<<<<<<<<<<<<<<<<<<<<<< drawFlurry >>>>>>>>>>>>>>>>>
void drawFlurry(int num, GLdouble maxx, GLdouble maxy,
	GLdouble windowx, GLdouble windowy)
// draw num random rectangles in a Width by Height rectangle 
{
	for (int i = 0; i < num; i++) 
	{
		GLdouble cx = random(windowx);
		GLdouble cy = random(windowy);
		GLdouble dx = random(0.5*maxx);
		GLdouble dy = random(0.5*maxy);
		glColor3d(randomdouble(),randomdouble(),randomdouble());
			// set the color level to random r, g, b vals between 0.0 and 1.0
		glRectd(cx-dx, cy+dy, cx+dx, cy-dy);	
			/* glRect takes two pairs of point coordinates, specifying
			two diagonally opposite corners of the rectangle.  Either
			pair is okay, either order of a given pair is okay, although
			this does affect the "winding".  We	use (left, bottom, 
			right, top) here, which produces counterclockwise winding. */
	}
}  
//<<<<<<<<<<<<<<<<<<<<<<<< myDisplay >>>>>>>>>>>>>>>>>
void myDisplay(void)
{
/* myDisplay is called once at start up and is called again whenever
the graphics window is first covered and then uncovered, making a
redraw necessary. */
	glClear(GL_COLOR_BUFFER_BIT);     // clear the screen 
	drawFlurry(20, 400, 300, MAXX, MAXY);
	glFlush();		                 // send all output to display 
}
//<<<<<<<<<<<<<<<<<<<<<<<< myAnimate >>>>>>>>>>>>>>>>>
void myAnimate(void)
{
/* myAnimate is called over and over whenever glut is idle (not busy
processing a key or mouse message). To keep this from happening too
often we put in a delay. */
	sleep(TIMEPERFRAME); //pause between updates, the sleep() arg is in seconds.
	myDisplay();
}
//<<<<<<<<<<<<<<<<<<<<<<<< myKeyboard >>>>>>>>>>>>>>>>>
void myKeyboard(unsigned char key, int mx, int my)
{
/* key is the ASCII code of the key, such as 'a', 'A', 'b', 'B', and
so on.  Note that case matters.  mx and my are the current mouse
position. */
	switch(key)
	{
		case ' ': //Space
			myDisplay(); //redraw 
			break;
		case 'q':
			exit(-1); //Terminate
			break;
	}
}
//<<<<<<<<<<<<<<<<<<<<<<<< myMouse >>>>>>>>>>>>>>>>>
void myMouse(int button, int state, int mx, int my)
{
/* Gets called when a button is clicked.  The arguments are:
button is GLUT_LEFT_BUTTON or GLUT_RIGHT_BUTTON,
state is GLUT_DOWN or GLUT_UP, 
and mx and my are window pixel coordinates, origin at window upper left. */
}
//<<<<<<<<<<<<<<<<<<<<<<<< myKeyboard >>>>>>>>>>>>>>>>>
void myMovedMouse(int mx, int my)
{
/*Gets called whenever the mouse moves, and mx and my are the new window
pixel coordinates, origin at window upper left. */
}
//<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>
void main(int argc, char** argv)
{
	glutInit(&argc, argv);          // initialize the toolkit
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB); // set display mode
	glutInitWindowSize(640,480);     // set window size
	glutInitWindowPosition(100, 150); // set window position on screen
	glutCreateWindow("Rectangle Flurry by Rudy Rucker, August 28, 2003 (See Hill,2.31)"); // open the screen window
	glutDisplayFunc(myDisplay);     // register redraw function
	glutKeyboardFunc(myKeyboard);	// register keyboard function.
	glutMouseFunc(myMouse);			// register mouse click function.
	glutMotionFunc(myMovedMouse);   // register mouse move function.
#ifdef AUTOANIMATE
	glutIdleFunc(myAnimate);		// register idle function to call.
#endif //AUTOANIMATE
	myInit();                   
	glutMainLoop(); 		     // go into a perpetual loop
}
